// -*- c++ -*-
//------------------------------------------------------------------------
// $Id: autoptr_test.cpp,v 1.2 2002/12/05 03:20:17 vlg Exp $
//------------------------------------------------------------------------
//                           src/test/autoptr.C
//------------------------------------------------------------------------------
//  Copyright (c) 1999 by Vladislav Grinchenko
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version
//  2 of the License, or (at your option) any later version.
//------------------------------------------------------------------------
//
// Description:
//
//	This tests and illustrates auto_ptr template defined in
//	assa/IsAutoPtr.h file
//
// Date: March 15, 1999
//------------------------------------------------------------------------

#include <iostream>
#include <assert.h>
#include "assa/AutoPtr.h"

using namespace std;
using namespace ASSA;

class Mess 
{
private:
    static int m_cnt;

public:
    Mess() { 
		cout << "-- Mess::Mess --\n";
		m_cnt++; 
    }
    ~Mess() { 
		cout << "-- Mess::~Mess --\n";
		m_cnt--; 
    }

    Mess (const Mess& /* m_ */) { 
		cout << "-- Mess::Mess(const Mess&) --\n";
		m_cnt++; 
    }
    
    Mess& operator= (const Mess& /* m_ */) { 
		cout << "-- Mess::operator= --\n";
		m_cnt++; 
		return *this;
    }

    static int getCount() { return m_cnt; }
    static void resetCount() { m_cnt = 0; }
    void printMsg() { cout << "I'm a Mess class!\n"; }
};

int Mess::m_cnt;

class MyMess : public Mess {};

void leak_mem ()
{
	cout << "-- leak_mem --\n";
	Mess* a = new Mess();

	a->printMsg();
}

void no_leak_mem ()
{
	cout << "-- no_leak_mem --\n";
	AutoPtr<Mess> a (new Mess);

	a->printMsg();
}

int main()
{
	cout << "\n*** Testing memory allocation/deallocation ***\n";

	cout << "\nCalling leak_mem()\n";
	leak_mem();
	cout << "copies on heap still allocated: " << Mess::getCount();
	assert (Mess::getCount() == 1);
	cout << " ... ok\n";

	Mess::resetCount ();

	cout << "\nCalling no_leak_mem()\n";
	no_leak_mem();
	cout << "copies on heap still allocated: " << Mess::getCount();
	assert (Mess::getCount() == 0);
	cout << " ... ok\n";

	// How about rebinding later in the game?

	AutoPtr<Mess> fp;
	bool cond = true;

	if (cond) 
	{
	    AutoPtr<Mess> temp (new Mess);
	    fp = temp;
	}

	//---------------------------------------------------------------

	cout << "\n*** Testing ownership transfer ***\n\n";

	AutoPtr<Mess> p1 (new Mess());
	AutoPtr<Mess> p2;

	cout << "\nUninitialized AutoPtr is a '0' pointer ...";
	assert (p2.get() == 0);
	cout << " ok\n";

	cout << "Ownership transfer by assignment ... ";
	p2 = p1;
	assert (p2.get() != 0);
	cout << "ok\n";

	cout << "Ownership transfer by copy constructor ... ";
	AutoPtr<Mess> p3(p2);

	assert (p3.get() != 0);
	cout << "ok\n";

// 	cout << "\n*** Testing inheritance handling ***\n\n";

// 	AutoPtr<Mess> pm ( new MyMess );
// 	AutoPtr<MyMess> pmym ( new MyMess );

// 	pm = pmym;
// 	AutoPtr<Mess> pm2 ( pmym );

// 	cout << "done!\n";

	cout << "\nTest done!\n\n";
}

